//------------------ VERSION MAIN 2 ------------------\\
//----------- STARTED JULY 24, 2012 2:50 PM ---------\\
//--------------- ATMEGA 328P 3.3V 8MHZ --------------\\

//--------------------------------------\\
//------------- LIBRARIES --------------\\

#include <Wire.h>
#include <EasyTransferI2C.h>

//--------------------------------------\\
//------------ EASYTRANSFER ------------\\

#define I2C_SLAVE_ADDRESS 5 // Address of slave I2C device

EasyTransferI2C ETI2C; // Object used for EasyTransfer I2C communication

struct RECEIVE_DATA_STRUCTURE // Set of data used in EasyTransfer communication
{
  int xAng;
  int yAng;
  int zAng;
  int xDeg;
  int yDeg;
  //int zDeg;
  float rX;
  float rY;
  //float rZ;
  float rRadius;
  int rDegree;
  int sendSpare;
};

RECEIVE_DATA_STRUCTURE accelData; // Name of data structure used in EasyTransfer communication

int ledPins[] = {4, A0, 13, 3, A1, A3, 2, A2, 7, 9, 10, 8, 6, 12, 11, 5}; // An array that holds the value of the pins connected to the LED display 

int numLeds = 16; // Number of indicator LEDs

int rDegreeMax = 360; // Maximum, minimum value of rDegree that will be recognized, used in map() function
int rDegreeMin = 1;

int rDegreeMap; // Mapped value of rDegree
int rDegreeMapMax = 16; // Maximum value of mapped rDegree, equal to the number of indicator LEDs
int rDegreeMapMin = 0;

int rDegreeDisplay; // Modified value of rDegree used for LED display

int rRadiusMax = .98;
int rRadiusMin = .07;

//float rRadiusInc; // Increased value of rRadius, just rRadius x 10
int rRadiusMap; // Mapped value of rRadius
int rRadiusMapMax = 8;
int rRadiusMapMin = 0;



//--------------------------------------\\
//--------------- SETUP ----------------\\

void setup()
{
  Serial.begin(9600); // OPTIONAL Initialize serial for debugging thru computer
  Wire.begin(I2C_SLAVE_ADDRESS);
  ETI2C.begin(details(accelData), &Wire);
  Wire.onReceive(receive);
  
  for (int thisPin = 0; thisPin < numLeds; thisPin++)
  {
    pinMode(ledPins[thisPin], OUTPUT);
  }
}

//--------------------------------------\\
//------------- MAIN LOOP --------------\\

void loop()
{
  if(ETI2C.receiveData())
  {
  
  //--------------------------------------\\
  //--------- LOCAL SERIAL DEBUG ---------\\
  
  /*
  Serial.print("   xAng= "); Serial.print(accelData.xAng);
  Serial.print("   yAng= "); Serial.print(accelData.yAng);
  Serial.print("   zAng= "); Serial.print(accelData.zAng);
  Serial.print("   xDeg= "); Serial.print(accelData.xDeg);
  Serial.print("   yDeg= "); Serial.print(accelData.yDeg);
  Serial.print("   rX= "); Serial.print(accelData.rX);
  Serial.print("   rY= "); Serial.print(accelData.rY);
  Serial.print("   rRadius= "); Serial.print(accelData.rRadius);
  Serial.print("   rDegree= "); Serial.print(accelData.rDegree);*/
  
  //--------------------------------------\\
  
  rDegreeMap = map(accelData.rDegree, rDegreeMin, rDegreeMax, rDegreeMapMin, rDegreeMapMax);
  Serial.print("   rDegreeMap="); Serial.print(rDegreeMap);
  Serial.print("   rDegree="); Serial.println(accelData.rDegree);
  
  /*rRadiusInc = (accelData.rRadius * 10.0);
  Serial.print("   rRadiusInc="); Serial.print(rRadiusInc);*/
  
  rRadiusMap = map((accelData.rRadius * 10.0), 1, 10, 1, 8);
  //Serial.print("   rRadiusMap="); Serial.println(rRadiusMap);
  
  //Serial.println("   ATmega");
  
  
  //--------------------------------------\\
  
  // LOTS OF TENTATIVE IF AND STATEMENTS FOR LED DISPLAY
  
  
  
  
  
  //--------------------------------------\\  
  
  for (int thisLed = 0; thisLed < numLeds; thisLed++)          
  {
    if(thisLed == rDegreeMap)
    {
        digitalWrite(ledPins[thisLed], HIGH);
        digitalWrite(ledPins[thisLed-1], HIGH);
        
       
        
        
    }
        
    else
    {
      digitalWrite(ledPins[thisLed], LOW);
     
      if(rDegreeMap == 0) // Necessary for far right LEDs to light up properly when tilted to far right. Accounts for jump between 0 and 360 degrees at end of direction loop 
        {
          digitalWrite(ledPins[15], HIGH);
        }
    }
  }
  
  
  /*
  if(accelData.rRadius <= 0.1)
  {
    for (int thisLed = 0; thisLed < numLeds; thisLed++)
    {
      digitalWrite(ledPins[thisLed], LOW);
    }
  }*/
  
  //--------------------------------------\\
  
  }
  else
  {
    //Serial.println("DATA NOT RECEIVED");
  }
}

void receive(int numBytes){}


