//----------------- VERSION MAIN 1 -------------------\\
//---------- STARTED JULY 23, 2012 11:00 PM ----------\\
//------------ ARDUINO PRO MINI 3.3V 8MHZ ------------\\

// http://www.starlino.com/imu_guide.html \\

//--------------------------------------\\
//------------- LIBRARIES --------------\\

#include <Wire.h>
#include <EasyTransfer.h>
#include <EasyTransferI2C.h>

//--------------------------------------\\
//------------ EASYTRANSFER ------------\\

#define I2C_SLAVE_ADDRESS 5 // Address of slave I2C device

EasyTransfer ETSerial; // Object used for EasyTransfer serial communication
EasyTransferI2C ETI2C; // Object used for EasyTransfer I2C communication

struct SEND_DATA_STRUCTURE // Set of data used in EasyTransfer communication
{
  int xAng;
  int yAng;
  int zAng;
  int xDeg;
  int yDeg;
  //int zDeg;
  float rX;
  float rY;
  //float rZ;
  float rRadius;
  int rDegree;
  int sendSpare;
};

SEND_DATA_STRUCTURE accelData; // Name of data structure used in EasyTransfer communication

//--------------------------------------\\
//-------------- VARIABLES -------------\\

const int xPin = 2; // Analog pins reading from accelerometer
const int yPin = 1;
const int zPin = 0;

int xRaw; // Raw values read from accelerometer
int yRaw;
int zRaw;

/*
int xVal; // Standard name for accelerometer values
int yVal;
int zVal;
*/

int xAng; // Used for figuring out degrees of tilt. From -90 to 90
int yAng;
int zAng;

int xDeg; // Degree of tilt on each axis. From 0 to 360
int yDeg;
int zDeg;

int accelRawMinVal = 400; // Min and Max values read from acclerometer. Used to figure out degrees of tilt
int accelRawMaxVal = 620;

float rX; // Amount of tilt on each axis measured in G's. From -1 to 0 , 0 to 1
float rY;
float rZ;

float rRadius; // 'Radius' of circle encompassing tilt. 'Distance' from center(0, 0) to current tilt point. From 0 to 1

int rDegree; // True direction of tilt. From 0 to 360

float vRef = 3.3; // Analog reference voltage

float vZeroG = 1.65; // Zero G accelerometer voltage

float accelSensitivity = 0.330; // Sensitivity of acclerometer

int adcRange = 1023; // ADC range of Arduino

int sendSpare;

//--------------------------------------\\
//--------------- SETUP ----------------\\

void setup()
{
  Serial.begin(9600); // Initialize serial
  Wire.begin(); // Initialize I2C
  ETSerial.begin(details(accelData), &Serial); // Initialize ET serial protocol 
  ETI2C.begin(details(accelData), &Wire); // Initialize ET I2C protocol
}

//--------------------------------------\\
//------------- MAIN LOOP --------------\\

void loop()
{
  
  //--------------------------------------\\
  //---------- ACCEL DATA CALC -----------\\
  
  int xRaw = analogRead(xPin); // Read raw values from accelerometer
  int yRaw = analogRead(yPin);
  int zRaw = analogRead(zPin);
  
  int xAng = ( ( -map(xRaw, accelRawMinVal, accelRawMaxVal, -90, 90) ) - 8 ); // Remap raw accel values from -90 to 90 //*MODIFIED 12/22/12 - ADDED '- ____' TO ACCOMODATE FOR ERROR IN ACCEL POSITIONING*
  int yAng = ( ( -map(yRaw, accelRawMinVal, accelRawMaxVal, -90, 90) ) - 8 ); //*MODIFIED 12/22/12 - ADDED '- ____' TO ACCOMODATE FOR ERROR IN ACCEL POSITIONING*     *THIS IS DUE TO MISTAKE IN SOLDERING*
  int zAng = -map(zRaw, accelRawMinVal, accelRawMaxVal, -90, 90);

  int yDeg = RAD_TO_DEG * (atan2(-yAng, -zAng) + PI); // Determine degrees of tilt on each axis from 0 to 360
  int xDeg = RAD_TO_DEG * (atan2(-xAng, -zAng) + PI);
  int zDeg = RAD_TO_DEG * (atan2(-yAng, -xAng) + PI);
  
  float rX = ( ( ( ( ( xRaw * vRef) / adcRange) - vZeroG) / accelSensitivity) + 0.26 ); // Convert raw accel data into g's
  float rY = ( ( ( ( ( yRaw * vRef) / adcRange) - vZeroG) / accelSensitivity) + 0.27 ); // *ADDITIONAL ' + ____ ' ADDED TO ACCOMODATE FOR ERROR IN ACCEL POSITIONING, IN ORDER TO TRULY 'CENTER' ACCEL
  float rZ = ( ( ( ( zRaw * vRef) / adcRange) - vZeroG) / accelSensitivity);                                                               // *THIS IS DUE TO MISTAKE IN SOLDERING*
  
  int rDegree = RAD_TO_DEG * (atan2(rY, rX) + (PI)); // Figure out the 'direction' / angle of the point determined by coordinates (rX, rY)
  
  float rRadius = sqrt ( ( pow(rX, 2) + pow(rY, 2) ) ); // Figure out the amount of tilt in a certain direction / the 'radius'

  //--------------------------------------\\
  //--------- LOCAL SERIAL DEBUG ---------\\
  
  /*                                                        // 'COMMENT OUT' SERIAL DEBUG CODE FOR FINAL PROGRAM, SERIAL DEBUG CODE SLOWS DOWN PROGRAM CONSIDERABLY
  Serial.print("rX= "); Serial.print(rX);
  Serial.print("   rY= "); Serial.print(rY);
  Serial.print("   rZ = "); Serial.print(rZ);
  Serial.print("   rRadius= "); Serial.print(rRadius);
  Serial.print("   xDeg= "); Serial.print(xDeg);
  Serial.print("   yDeg= "); Serial.print(yDeg);
  Serial.print("   zDeg= "); Serial.print(zDeg);
  Serial.print("   xAng= "); Serial.print(xAng);
  Serial.print("   yAng= "); Serial.print(yAng);
  Serial.print("   zAng= "); Serial.print(zAng);
  Serial.print("   rDegree= "); Serial.println(rDegree);
  /*Serial.print("   xRaw= "); Serial.print(xRaw);
  Serial.print("   yRaw= "); Serial.print(yRaw);
  Serial.print("   zRaw= "); Serial.println(zRaw);*/
  //Serial.println("   PRO MINI");
  

  //--------------------------------------\\
  //------------- SEND DATA --------------\\  
  
  
  accelData.xAng = xAng; // Drop all calculated values into EasyTransfer data
  accelData.yAng = yAng;
  accelData.zAng = zAng;
  accelData.xDeg = xDeg;
  accelData.yDeg = yDeg;
  //accelData.zDeg = zDeg;
  accelData.rX = rX;
  accelData.rY = rY;
  //accelData.rZ = rZ;
  accelData.rRadius = rRadius;
  accelData.rDegree = rDegree;
  accelData.sendSpare = sendSpare;
  
  ETI2C.sendData(I2C_SLAVE_ADDRESS); // Send ET data over I2C  
  ETSerial.sendData(); // Send ET data over serial
  delay(50);                                                              // * WATCH THIS DELAY, IF MALFUNCITON, INCREASE DELAY, VERY IMPORTANT
  

}

